@@ -149,6 +149,9 @@ static GdkFilterReturn hildon_desktop_x_event_filter (GdkXEvent *xevent, GdkEven
 static void hildon_desktop_panel_embed_applet (HildonDesktopPanelExpandable *panel, Window wid);
 #endif
 
+static gint
+hildon_desktop_panel_expandable_sort_items (gconstpointer a, gconstpointer b);
+
 static GList *
 hildon_desktop_panel_expandable_get_children (HildonDesktopContainer *container);
 
@@ -460,16 +463,19 @@ hildon_desktop_panel_expandable_get_children (HildonDesktopContainer *container)
 		  	(GHFunc)hildon_desktop_panel_expandable_hash_adding,
 			(gpointer)&retval);
 
+  retval =
+    g_list_sort (retval,(GCompareFunc)hildon_desktop_panel_expandable_sort_items);
+
+  retval = g_list_reverse (retval);
+  
   return retval;
 }	
 
 static void 
 hildon_desktop_panel_expandable_cadd (GtkContainer *container, GtkWidget *widget)
 {
-  GList *children;	
-  g_return_if_fail (HILDON_DESKTOP_IS_PANEL_EXPANDABLE (container));
-
-  children = gtk_container_get_children (container);
+  g_return_if_fail (HILDON_DESKTOP_IS_PANEL_EXPANDABLE (container) &&
+                    GTK_IS_WIDGET (widget));
 
   if (widget == HILDON_DESKTOP_PANEL_EXPANDABLE (container)->priv->arrow)
     return;	  
