@@ -113,6 +113,7 @@ static void hildon_desktop_popup_menu_select_next_prev_item (HildonDesktopPopupM
 static void hildon_desktop_popup_menu_scroll_start (GtkWidget *widget, HildonDesktopPopupMenu *menu);
 static void hildon_desktop_popup_menu_scroll_stop (GtkWidget *widget, HildonDesktopPopupMenu *menu);
 static void hildon_desktop_popup_menu_adjustment_changed (GtkAdjustment *adj, HildonDesktopPopupMenu *menu);
+static void hildon_desktop_popup_menu_box_remove (HildonDesktopPopupMenu *menu, GtkWidget *widget);
 
 static void 
 hildon_desktop_popup_menu_init (HildonDesktopPopupMenu *menu)
@@ -299,6 +300,10 @@ hildon_desktop_popup_menu_constructor (GType gtype,
   gtk_widget_push_composite_child ();
 
   menu->priv->box_items = gtk_vbox_new (FALSE, 0); /* FIXME: add spacing decoration */
+
+  g_signal_connect_swapped (menu->priv->box_items, "remove",
+                            G_CALLBACK (hildon_desktop_popup_menu_box_remove),
+                            menu);
   
   gtk_widget_set_size_request (menu->priv->box_items, 1, -1);
 
@@ -557,7 +562,8 @@ hildon_desktop_popup_menu_motion_notify (GtkWidget      *widget,
       if (hildon_desktop_popup_menu_item_is_selectable (GTK_WIDGET (l->data)) &&
           menu->priv->selected_item != GTK_MENU_ITEM (l->data))
       {
-        gtk_item_deselect (GTK_ITEM (menu->priv->selected_item));
+        if (GTK_IS_ITEM (menu->priv->selected_item))
+          gtk_item_deselect (GTK_ITEM (menu->priv->selected_item));
         gtk_item_select (GTK_ITEM (l->data));
         menu->priv->selected_item = GTK_MENU_ITEM (l->data);
       }
@@ -716,7 +722,8 @@ hildon_desktop_popup_menu_press_event (GtkWidget      *widget,
         if (hildon_desktop_popup_menu_item_is_selectable (GTK_WIDGET (l->data)) &&
             menu->priv->selected_item != GTK_MENU_ITEM (l->data))
         {
-          gtk_item_deselect (GTK_ITEM (menu->priv->selected_item));
+          if (GTK_IS_ITEM (menu->priv->selected_item))
+            gtk_item_deselect (GTK_ITEM (menu->priv->selected_item));
           gtk_item_select (GTK_ITEM (l->data));
           menu->priv->selected_item = GTK_MENU_ITEM (l->data);
         }
@@ -989,12 +996,24 @@ hildon_desktop_popup_menu_real_add_item (HildonDesktopPopupMenu *menu,
 static void
 hildon_desktop_popup_menu_real_remove_item (HildonDesktopPopupMenu *menu, GtkMenuItem *item)
 {
-  if (menu->priv->selected_item == item)
-    hildon_desktop_popup_menu_deselect_item (menu, item);
-
   gtk_container_remove (GTK_CONTAINER (menu->priv->box_items), GTK_WIDGET (item));
- 
-  menu->priv->n_items--;
+
+}
+
+static void
+hildon_desktop_popup_menu_box_remove (HildonDesktopPopupMenu *menu,
+                                      GtkWidget *widget)
+{
+  if (GTK_IS_MENU_ITEM (widget))
+  {
+    GtkMenuItem *item = (GtkMenuItem *)widget;
+
+    if (menu->priv->selected_item == item)
+      hildon_desktop_popup_menu_deselect_item (menu, item);
+
+    menu->priv->n_items--;
+  }
+
 }
 
 void
@@ -1044,6 +1063,7 @@ hildon_desktop_popup_menu_replace_items (HildonDesktopPopupMenu *menu, GList *it
     
     hildon_desktop_popup_menu_real_remove_item (menu, GTK_MENU_ITEM (item));
   }
+  g_list_free (children);
  
   for (l = items; l != NULL; l = l->next)
   {
@@ -1136,7 +1156,10 @@ hildon_desktop_popup_menu_select_first_item (HildonDesktopPopupMenu *menu)
       break;
     }	    
   }
-}	
+
+  if (children)
+    g_list_free (children);
+}
 
 void
 hildon_desktop_popup_menu_select_last_item (HildonDesktopPopupMenu *menu)
@@ -1161,7 +1184,10 @@ hildon_desktop_popup_menu_select_last_item (HildonDesktopPopupMenu *menu)
       break;
     }
   }
-}	
+
+  if (children)
+    g_list_free (children);
+}
 
 void
 hildon_desktop_popup_menu_activate_item (HildonDesktopPopupMenu *menu, GtkMenuItem *item)
@@ -1181,7 +1207,8 @@ hildon_desktop_popup_menu_activate_item (HildonDesktopPopupMenu *menu, GtkMenuIt
     }
   }
 
-  g_list_free (children);				  
+  if (children)
+    g_list_free (children);				  
 }	
 
 GtkMenuItem *
