@@ -66,6 +66,10 @@ void
 hildon_desktop_popup_menu_remove_item (HildonDesktopPopupMenu *menu,
 				       GtkMenuItem            *item);
 
+void
+hildon_desktop_popup_menu_replace_items (HildonDesktopPopupMenu *menu, 
+		                         GList *items);
+
 GList *
 hildon_desktop_popup_menu_get_children (HildonDesktopPopupMenu *menu);
 
