@@ -33,6 +33,7 @@
 #include <gtk/gtk.h>
 #include <libgnomevfs/gnome-vfs.h>
 #include <glib/gi18n.h>
+#include <glib/gstdio.h>
 
 #ifdef HAVE_LIBOSSO
 #include <libosso.h>
@@ -114,6 +115,9 @@ struct _HDDesktopPrivate
 #ifdef HAVE_LIBOSSO
   osso_context_t  *osso_context;
 #endif
+#ifdef HAVE_SAFE_MODE
+  gboolean               safe_mode;
+#endif
 };
 
 static void hd_desktop_load_containers (HDDesktop *desktop);
@@ -373,8 +377,12 @@ static GList *
 hd_desktop_plugin_list_from_container (GtkContainer *container)
 {
   GList *plugin_list = NULL, *children, *iter;
-
-  children = gtk_container_get_children (container);
+  
+  if (HILDON_DESKTOP_IS_CONTAINER (container))
+    children = 
+      hildon_desktop_container_get_children (HILDON_DESKTOP_CONTAINER (container));
+  else	  
+    children = gtk_container_get_children (container);
 
   for (iter = children; iter; iter = g_list_next (iter))
   {
@@ -479,6 +487,7 @@ hd_desktop_plugin_list_from_conf (const gchar *config_file)
   if (error)
   {
     g_warning ("Error loading container configuration file %s: %s", config_file, error->message);
+    g_key_file_free (keyfile);
     g_error_free (error);
     
     return NULL;
@@ -500,11 +509,11 @@ hd_desktop_plugin_list_from_conf (const gchar *config_file)
       error = NULL;
     }      
 
-    if (is_to_load)    
-      plugin_list = g_list_append (plugin_list, groups[i]);
+    if (is_to_load) 
+      plugin_list = g_list_append (plugin_list, g_strdup (groups[i]));
   }
 
-  g_free (groups);
+  g_strfreev (groups);
   g_key_file_free (keyfile);
 
   return plugin_list;
@@ -581,7 +590,13 @@ hd_desktop_select_plugins (HildonDesktopWindow *window, gpointer user_data)
 
   priv = info->desktop->priv;
 
-  loaded_plugins = gtk_container_get_children (window->container);
+  if (HILDON_DESKTOP_IS_CONTAINER (window->container))
+    loaded_plugins = 
+      hildon_desktop_container_get_children 
+        (HILDON_DESKTOP_CONTAINER (window->container));
+  else
+    loaded_plugins = 
+      gtk_container_get_children (window->container);
 
   response = hd_select_plugins_dialog_run (loaded_plugins,
 #ifdef HAVE_LIBHILDONHELP
@@ -903,18 +918,30 @@ hd_desktop_load_containers (HDDesktop *desktop)
       g_error_free (error);
       error = NULL;
     }
-    
-    load_new_plugins = g_key_file_get_boolean (keyfile, 
-                                               groups[i], 
-                                               HD_DESKTOP_CONFIG_KEY_LOAD_NEW_PLUGINS,
-                                               &error);
-
-    if (error)
+#ifdef HAVE_SAFE_MODE    
+    /* Check for safe mode */
+    if (priv->safe_mode)
     {
+      /* Do not load new plugins automatically in safe mode ever */
       load_new_plugins = FALSE;
-      g_error_free (error);
-      error = NULL;
+    } 
+    else 
+    {
+#endif
+      load_new_plugins = g_key_file_get_boolean (keyfile, 
+                                                 groups[i], 
+                                                 HD_DESKTOP_CONFIG_KEY_LOAD_NEW_PLUGINS,
+                                                 &error);
+
+      if (error)
+      {
+        load_new_plugins = FALSE;
+        g_error_free (error);
+        error = NULL;
+      }
+#ifdef HAVE_SAFE_MODE
     }
+#endif
     
     policy_module = g_key_file_get_string (keyfile, 
                                            groups[i], 
@@ -1288,11 +1315,31 @@ hd_desktop_load_containers (HDDesktop *desktop)
 
     plugin_list = hd_desktop_plugin_list_from_conf (container_config);
 
-    hd_plugin_manager_load (HD_PLUGIN_MANAGER (priv->pm), 
-                            plugin_list, 
-                            HILDON_DESKTOP_WINDOW (info->container)->container,
-			    info->policy);
-
+#ifdef HAVE_SAFE_MODE
+    if (priv->safe_mode)
+    {
+      /* 
+       * Load only plugins according to policy by giving hd_plugin_manager_load
+       * NULL (==empty list) instead of GList plugin_list. This supposes that
+       * the default plugins within the policy are well behaving and do not cause
+       * crash-loop which this mechanism was made to prevent to happen. 
+       */
+      hd_plugin_manager_load (HD_PLUGIN_MANAGER (priv->pm), 
+                              NULL, 
+                              HILDON_DESKTOP_WINDOW (info->container)->container,
+                              info->policy);
+    } 
+    else 
+    {
+#endif
+      hd_plugin_manager_load (HD_PLUGIN_MANAGER (priv->pm), 
+                              plugin_list, 
+                              HILDON_DESKTOP_WINDOW (info->container)->container,
+                              info->policy);
+#ifdef HAVE_SAFE_MODE
+    }
+#endif
+    
     gtk_widget_show (info->container);
 
     g_free (type);
@@ -1683,6 +1730,41 @@ hd_desktop_init (HDDesktop *desktop)
 			         NULL);
 
   desktop->priv->dialog_queue = g_queue_new ();
+
+#ifdef HAVE_SAFE_MODE
+  const gchar *dev_mode = g_getenv ("SBOX_PRELOAD");
+
+  priv->safe_mode = FALSE;
+  
+  if (!dev_mode)
+  {
+    /* 
+     * Check for safe mode. The stamp file is created here and
+     * Removed in main after gtk_main by g_object_unref in a call to finalize
+     * function of this gobject in case of clean non-crash exit 
+     * Added by Karoliina <karoliina.t.salminen@nokia.com> 31.7.2007 
+     */
+    if (g_file_test (HILDON_DESKTOP_STAMP_FILE, G_FILE_TEST_EXISTS)) 
+    {
+      /* Hildon Desktop enters safe mode */
+      g_warning ("hildon-desktop did not exit properly on the previous "
+                 "session. All home and statusbar plugins will be disabled.");
+
+      priv->safe_mode = TRUE;
+    } 
+    else 
+    {
+      g_debug ("LALALA");
+            
+      /* Hildon Desktop enters normal mode and creates the stamp to track crashes */
+      g_mkdir_with_parents (HILDON_DESKTOP_STAMP_DIR, 0755);
+
+      g_file_set_contents (HILDON_DESKTOP_STAMP_FILE, "1", 1, NULL);
+
+      priv->safe_mode = FALSE;
+    }
+  }
+#endif
 }
 
 static void
