@@ -89,6 +89,8 @@
 #define AS_TEXT_COLOR              "PluginMenuTextColor"
 #define AS_SECONDARY_TEXT_COLOR    "PluginMenuSecondaryTextColor"
 
+#undef DEBUG
+
 /*
  * HDSwitcherMenuItem
  */
@@ -209,6 +211,8 @@ hd_switcher_menu_item_set_property (GObject      *gobject,
       
     case MENU_PROP_NOT_ICON:
       priv->notification_icon = g_value_get_object (value);
+      if (priv->notification_icon)
+        g_object_ref (priv->notification_icon);
       break;
       
     case MENU_PROP_DBUS_CALLBACK:
@@ -275,7 +279,7 @@ hd_switcher_menu_item_get_property (GObject    *gobject,
 
 static void
 hd_switcher_menu_item_icon_animation (GtkWidget *icon,
-	       			 gboolean   is_on)
+                                      gboolean   is_on)
 {
   GdkPixbuf *pixbuf;
   GdkPixbufAnimation *pixbuf_anim;
@@ -304,7 +308,8 @@ hd_switcher_menu_item_icon_animation (GtkWidget *icon,
 
     gtk_image_set_from_pixbuf (GTK_IMAGE (icon), pixbuf);
 
-    g_object_unref (pixbuf);
+    
+
   }
 }
 
@@ -346,6 +351,7 @@ hd_switcher_menu_item_constructor (GType                  type,
                                    GTK_RC_FG,
                                    GTK_STATE_NORMAL,
                                    AS_SECONDARY_TEXT_COLOR);
+  gtk_label_set_ellipsize (GTK_LABEL (priv->label2), PANGO_ELLIPSIZE_END);
   gtk_widget_set_composite_name (GTK_WIDGET (priv->label2), "as-app-menu-label-desc");
   gtk_misc_set_alignment (GTK_MISC (priv->label2), 0.0, 0.1);
   gtk_box_pack_end (GTK_BOX (vbox), priv->label2, TRUE, TRUE, 0);
@@ -356,6 +362,7 @@ hd_switcher_menu_item_constructor (GType                  type,
                                    GTK_RC_FG,
                                    GTK_STATE_NORMAL,
                                    AS_TEXT_COLOR);
+  gtk_label_set_ellipsize (GTK_LABEL (priv->label), PANGO_ELLIPSIZE_END);
   gtk_widget_set_composite_name (GTK_WIDGET (priv->label), "as-app-menu-label");
   gtk_misc_set_alignment (GTK_MISC (priv->label), 0.0, 0.9);
   gtk_box_pack_end (GTK_BOX (vbox), priv->label, TRUE, TRUE, 0);
@@ -467,7 +474,9 @@ hd_switcher_menu_item_constructor (GType                  type,
       gtk_image_set_from_pixbuf 
         (GTK_IMAGE (priv->icon), priv->notification_icon);
 
+#ifdef DEBUG
     g_debug ("pixbuf: %p Id: %d Summary of notification: %s", priv->notification_icon, priv->notification_id, priv->notification_summary);
+#endif
     
     gtk_label_set_text (GTK_LABEL (priv->label), 
 		        priv->notification_summary); /* TODO: Insert timestamp */
@@ -481,6 +490,8 @@ hd_switcher_menu_item_constructor (GType                  type,
       gtk_image_set_from_pixbuf 
         (GTK_IMAGE (priv->icon), priv->notification_icon);
 
+    gtk_misc_set_alignment (GTK_MISC (priv->label), 0.0, 0.5);
+
     gtk_label_set_text (GTK_LABEL (priv->label), 
 		        priv->notification_summary);
  
@@ -548,9 +559,6 @@ hd_switcher_menu_item_size_request (GtkWidget      *widget,
   if (child_width > label_width_ellipsize)
   {
     requisition->width = label_width_ellipsize;
-
-    gtk_label_set_ellipsize (GTK_LABEL (priv->label), PANGO_ELLIPSIZE_END);
-    gtk_label_set_ellipsize (GTK_LABEL (priv->label2), PANGO_ELLIPSIZE_END);
   }
   else
     requisition->width = MAX (requisition->width, child_width);
@@ -567,8 +575,9 @@ hd_switcher_menu_item_activate (GtkMenuItem *menu_item)
 
   if (info != NULL)
   {
+#ifdef DEBUG
     g_debug ("Raising application '%s'", hd_wm_entry_info_peek_title (info));
-  
+#endif 
     hd_wm_top_item (info);
   }
   else
@@ -624,9 +633,9 @@ hd_switcher_menu_item_button_release_event (GtkWidget      *widget,
 {
   HDSwitcherMenuItem *menuitem = HD_SWITCHER_MENU_ITEM(widget);
   gint x, y;
-  
+#ifdef DEBUG 
   g_debug ("menu item clicked ended");
-  
+#endif
   if(!menuitem->priv->show_close ||
      !menuitem->priv->close)
     return FALSE;
@@ -693,9 +702,9 @@ hd_switcher_menu_item_button_press_event (GtkWidget      *widget,
 {
   HDSwitcherMenuItem *menuitem = HD_SWITCHER_MENU_ITEM(widget);
   gint x, y;
-  
+#ifdef DEBUG 
   g_debug ("menu item clicked");
-  
+#endif
   if(!menuitem->priv->show_close ||
      !menuitem->priv->close)
     return FALSE;
@@ -706,7 +715,7 @@ hd_switcher_menu_item_button_press_event (GtkWidget      *widget,
    */
   
   gtk_widget_get_pointer(widget, &x, &y);
-
+#ifdef DEBUG
   g_debug ("pointer [%d,%d],\n"
           "close allocation [%d, %d, %d, %d]",
           x, y,
@@ -714,7 +723,7 @@ hd_switcher_menu_item_button_press_event (GtkWidget      *widget,
           menuitem->priv->close->allocation.y,
           menuitem->priv->close->allocation.width,
           menuitem->priv->close->allocation.height);
-
+#endif
   /* only test x here; y is always withing the button range */
   if(x >  menuitem->priv->close->allocation.x &&
      x <= menuitem->priv->close->allocation.x +
@@ -966,6 +975,7 @@ hd_switcher_menu_item_set_entry_info (HDSwitcherMenuItem *menuitem,
 {
   HDSwitcherMenuItemPrivate *priv;
   GdkPixbuf *pixbuf = NULL;
+  gchar *window_name = NULL, *app_name = NULL;
   
   g_return_if_fail (HD_IS_SWITCHER_MENU_ITEM (menuitem));
   g_return_if_fail (info != NULL);
@@ -1032,12 +1042,22 @@ hd_switcher_menu_item_set_entry_info (HDSwitcherMenuItem *menuitem,
  
     g_object_unref (pixbuf);
   }
-  
-  gtk_label_set_text (GTK_LABEL (priv->label),
-		      hd_wm_entry_info_peek_title (priv->info));
 
-  gtk_label_set_text (GTK_LABEL (priv->label2), 
-                      hd_wm_entry_info_get_window_name (priv->info));
+  app_name = hd_wm_entry_info_get_app_name (priv->info);
+  if (app_name)
+  {
+    gtk_label_set_text (GTK_LABEL (priv->label),
+                        app_name);
+    g_free (app_name);
+  }
+
+  window_name = hd_wm_entry_info_get_window_name (priv->info);
+  if (window_name)
+  {
+    gtk_label_set_text (GTK_LABEL (priv->label2),
+                        window_name);
+    g_free (window_name);
+  }
 
   g_object_notify (G_OBJECT (menuitem), "entry-info");
 }
