@@ -153,6 +153,8 @@ struct _HDSwitcherMenuPrivate
 
   GtkImage		   *image_button;
 
+  GtkWidget                *home_item;
+  
   GtkWidget 		   *active_menu_item;
   GtkWidget		   *clear_events_menu_item;
 
@@ -200,6 +202,7 @@ static void hd_switcher_menu_scroll_to (HildonDesktopPopupWindow *window,
 
 static void hd_switcher_menu_create_clear_notifications_button (HDSwitcherMenu *switcher);
 
+static void hd_switcher_menu_create_applications_menu (HDSwitcherMenu *switcher, HDWM *hdwm);
 static void hd_switcher_menu_refresh_application_items (HDSwitcherMenu *switcher);
 static void hd_switcher_menu_refresh_notification_items (HDSwitcherMenu *switcher);
 
@@ -490,6 +493,8 @@ hd_switcher_menu_popup_window_pane_keypress_cb (GtkWidget      *widget,
       (switcher->priv->menu_applications,
        GTK_MENU_ITEM (last->data));
 
+    g_list_free (applications);
+
     return TRUE;
   } 
 
@@ -606,6 +611,8 @@ hd_switcher_menu_update_open (HildonDesktopPopupWindow *window,
       }
     }
   }
+
+  g_list_free (children);
 #endif
 
   hd_switcher_menu_changed_stack_cb (switcher->hdwm, 
@@ -711,6 +718,9 @@ hd_switcher_menu_add_notification_group (gpointer key,
     menu_item = hd_switcher_menu_item_new_from_notification_group
       (ngroup->notifications, icon, summary, ngroup->dbus_callback, TRUE);
 
+    if (icon)
+      g_object_unref (icon);
+
     gtk_widget_set_name (GTK_WIDGET (menu_item), 
       		         HD_SWITCHER_MENU_NOT_MENU_ITEM_NAME);
 
@@ -862,8 +872,6 @@ hd_switcher_menu_display_state_cb (osso_display_state_t state, gpointer data)
 
   system_inactivity = (state == OSSO_DISPLAY_OFF); 
 
-  g_debug ("LALALALALALALALA: %d", system_inactivity);
-  
   if (system_inactivity != priv->system_inactivity)
   {
     priv->system_inactivity = system_inactivity;
@@ -1128,21 +1136,20 @@ hd_switcher_menu_constructor (GType gtype,
   
   gtk_widget_pop_composite_child ();
 
-  GtkWidget *menu_item = 
+  switcher->priv->home_item =
     hd_switcher_menu_item_new_from_entry_info 
       (hd_wm_get_home_info (hdwm), FALSE);
-
-  gtk_widget_set_name (GTK_WIDGET (menu_item), 
+  
+  gtk_widget_set_name (GTK_WIDGET (switcher->priv->home_item), 
       		       HD_SWITCHER_MENU_APP_MENU_ITEM_NAME);
 
-  hildon_desktop_popup_menu_add_item
-   (switcher->priv->menu_applications, GTK_MENU_ITEM (menu_item));
-
-  g_signal_connect_after (menu_item,
+  g_signal_connect_after (switcher->priv->home_item,
                           "activate",
                           G_CALLBACK (hd_switcher_menu_item_activated),
-                          switcher); 
+                          switcher);
 
+  hd_switcher_menu_create_applications_menu (switcher, hdwm);
+  
   switcher->priv->notification_groups = 
           g_hash_table_new_full (g_str_hash, 
 	  		         g_str_equal,
@@ -1224,6 +1231,12 @@ hd_switcher_menu_finalize (GObject *object)
   if (switcher->priv->window_dialog)
     gtk_widget_destroy (switcher->priv->window_dialog);	  
 
+  if (switcher->priv->home_item)
+  {
+    g_object_unref (switcher->priv->home_item);
+    switcher->priv->home_item = NULL;
+  }
+     
   g_object_unref (switcher->priv->icon_theme);
 
   hn_as_sound_deinit (switcher->priv->esd_socket); 
@@ -1261,6 +1274,15 @@ hd_switcher_menu_clear_item_activated (GtkMenuItem *menuitem, HDSwitcherMenu *sw
   GtkTreeIter iter;
   gboolean need_confirmation = FALSE;
 
+  if (switcher->priv->is_open)
+    hildon_desktop_popup_window_popdown
+      (switcher->priv->popup_window);
+
+  switcher->priv->is_open = FALSE;
+  
+  gtk_toggle_button_set_active
+    (GTK_TOGGLE_BUTTON (switcher->priv->toggle_button), FALSE);
+  
   nm = GTK_TREE_MODEL (switcher->nm);
 
   if (gtk_tree_model_get_iter_first (nm, &iter))
@@ -1296,8 +1318,11 @@ hd_switcher_menu_clear_item_activated (GtkMenuItem *menuitem, HDSwitcherMenu *sw
       hd_switcher_menu_confirm_clear_events ())
   {
     hildon_desktop_notification_manager_close_all (switcher->nm);
-    gtk_widget_destroy (switcher->priv->clear_events_menu_item);
-    switcher->priv->clear_events_menu_item = NULL;
+    if (switcher->priv->clear_events_menu_item)
+    {
+      gtk_widget_destroy (switcher->priv->clear_events_menu_item);
+      switcher->priv->clear_events_menu_item = NULL;
+    }
   }
 }
 	
@@ -1365,6 +1390,7 @@ hd_switcher_menu_replace_blinking_icon (HDSwitcherMenu *switcher, GdkPixbuf *ico
 
     image_blinking = gtk_image_new ();
     gtk_image_set_from_animation (GTK_IMAGE (image_blinking), icon_blinking);
+    g_object_unref (icon_blinking);
 
     if (image_blinking)
     {    
@@ -1604,6 +1630,8 @@ hd_switcher_menu_position_func (HildonDesktopPopupWindow  *menu,
 static void 
 hd_switcher_menu_toggled_cb (GtkWidget *button, HDSwitcherMenu *switcher)
 {
+  GList *children;
+  
   if (!GTK_WIDGET_VISIBLE (GTK_BIN (switcher->priv->toggle_button)->child))
     return;
 
@@ -1623,10 +1651,15 @@ hd_switcher_menu_toggled_cb (GtkWidget *button, HDSwitcherMenu *switcher)
     (gpointer)switcher,
     GDK_CURRENT_TIME);
 
-  if (!hildon_desktop_popup_menu_get_children (switcher->priv->menu_notifications))
+  children = hildon_desktop_popup_menu_get_children (switcher->priv->menu_notifications);
+  
+  if (!children)
     gtk_widget_hide (switcher->priv->notifications_window);
   else
+  {
     gtk_widget_show (switcher->priv->notifications_window);  
+    g_list_free (children);
+  }
 
   switcher->priv->is_open = TRUE;
 
@@ -1794,11 +1827,19 @@ hd_switcher_menu_create_notifications_menu (HDSwitcherMenu *switcher)
 
         if (!ack && !icon_is_set)
         {
+          GdkPixbuf *icon;
+
 	  switcher->priv->blinking_notification = id;
 
-          hd_switcher_menu_replace_blinking_icon (switcher, 
-			  			  hd_switcher_menu_get_icon_from_theme (icon_name,
-											AS_ICON_SIZE));
+          icon = hd_switcher_menu_get_icon_from_theme (icon_name,
+                                                       AS_ICON_SIZE);
+          if (icon)
+          {
+            hd_switcher_menu_replace_blinking_icon (switcher,
+                                                    icon);
+
+            g_object_unref (icon);
+          }
 
           icon_is_set = TRUE;
 	}
@@ -1807,6 +1848,8 @@ hd_switcher_menu_create_notifications_menu (HDSwitcherMenu *switcher)
 	  first_item = menu_item;
       }
 
+      if (icon)
+        g_object_unref (icon);
       g_free (icon_name);
       g_free (summary);
       g_free (body);
@@ -1828,29 +1871,12 @@ hd_switcher_menu_create_notifications_menu (HDSwitcherMenu *switcher)
 static void 
 hd_switcher_menu_create_applications_menu (HDSwitcherMenu *switcher, HDWM *hdwm)
 {
-  GList *children = NULL, *apps = NULL, *l;
+  GList *items = NULL, *apps = NULL, *l;
   GtkWidget *separator = NULL;
 
-  children =
-    hildon_desktop_popup_menu_get_children (switcher->priv->menu_applications);
-
-  for (l = children; l != NULL; l = g_list_next (l))
-  {
-    HDWMEntryInfo *info;
-
-    if (GTK_IS_SEPARATOR_MENU_ITEM (l->data))
-      info = NULL;
-    else 
-      info = 
-        hd_switcher_menu_item_get_entry_info 
-	  (HD_SWITCHER_MENU_ITEM (l->data));
+  g_object_ref (switcher->priv->home_item);
  
-    if (info != hd_wm_get_home_info (switcher->hdwm))
-      hildon_desktop_popup_menu_remove_item (switcher->priv->menu_applications,
-                                             l->data);
-  }
-
-  g_list_free (children);
+  items = g_list_append (items, switcher->priv->home_item);
 
   apps = g_list_reverse (g_list_copy (hd_wm_get_applications (hdwm)));
 
@@ -1866,8 +1892,7 @@ hd_switcher_menu_create_applications_menu (HDSwitcherMenu *switcher, HDWM *hdwm)
     {
       separator = hd_switcher_menu_create_separator (TRUE);
 
-      hildon_desktop_popup_menu_add_item
-       (switcher->priv->menu_applications, GTK_MENU_ITEM (separator));
+      items = g_list_append (items, separator);
     }
 
     for (child = children; child != NULL; child = child->next)
@@ -1884,8 +1909,7 @@ hd_switcher_menu_create_applications_menu (HDSwitcherMenu *switcher, HDWM *hdwm)
 			      G_CALLBACK (hd_switcher_menu_item_activated),
 			      (gpointer)switcher);
 
-      hildon_desktop_popup_menu_add_item
-       (switcher->priv->menu_applications, GTK_MENU_ITEM (menu_item));
+      items = g_list_append (items, menu_item);
 
       if (hd_wm_entry_info_is_active (entry))
         hildon_desktop_popup_menu_select_item 
@@ -1899,13 +1923,16 @@ hd_switcher_menu_create_applications_menu (HDSwitcherMenu *switcher, HDWM *hdwm)
     {
       separator = hd_switcher_menu_create_separator (TRUE);
 
-      hildon_desktop_popup_menu_add_item
-        (switcher->priv->menu_applications, GTK_MENU_ITEM (separator));
+      items = g_list_append (items, separator);
     }
 
     g_list_free (children);
   }
 
+  hildon_desktop_popup_menu_replace_items (switcher->priv->menu_applications, 
+		                           items);
+ 
+  g_list_free (items);
   g_list_free (apps);
 
   hd_switcher_menu_check_content (switcher); 
@@ -2047,6 +2074,8 @@ hd_switcher_menu_refresh_application_items (HDSwitcherMenu *switcher)
       }
     }
   } 
+
+  g_list_free (children);
 }
 
 static void
@@ -2075,6 +2104,8 @@ hd_switcher_menu_refresh_notification_items (HDSwitcherMenu *switcher)
     hd_switcher_menu_item_set_blinking (HD_SWITCHER_MENU_ITEM (l->data), 
 		    			!ack && !switcher->priv->system_inactivity);
   }
+
+  g_list_free (children);
 }
 
 static void 
@@ -2147,6 +2178,9 @@ hd_switcher_menu_changed_info_cb (HDWM *hdwm,
     if (pos > SWITCHER_N_SLOTS)
       make_it_blink = TRUE;	     
   }	  
+
+  if (switcher->priv->fullscreen)
+    make_it_blink = TRUE;	  
 	  
   children =
     hildon_desktop_popup_menu_get_children (switcher->priv->menu_applications);
@@ -2171,7 +2205,7 @@ hd_switcher_menu_changed_info_cb (HDWM *hdwm,
   if (menu_item)
   {
     hd_switcher_menu_item_set_entry_info (HD_SWITCHER_MENU_ITEM (menu_item), info);
-  }   
+  }
   
   if (!hd_wm_entry_info_is_urgent (info) &&
        hd_wm_entry_info_get_ignore_urgent (info))
@@ -2193,8 +2227,7 @@ hd_switcher_menu_changed_info_cb (HDWM *hdwm,
     if (menu_item)
     {
       /* child of one of the app buttons */
-      if (!hd_switcher_menu_item_is_blinking (HD_SWITCHER_MENU_ITEM (menu_item)) &&
-          !switcher->priv->system_inactivity)
+      if (!switcher->priv->system_inactivity)
       {
         hd_switcher_menu_item_set_blinking (HD_SWITCHER_MENU_ITEM (menu_item), TRUE);
       }
@@ -2213,6 +2246,14 @@ hd_switcher_menu_changed_info_cb (HDWM *hdwm,
   {
     hd_switcher_menu_item_set_blinking (HD_SWITCHER_MENU_ITEM (menu_item), FALSE);
   }
+
+  if (make_it_blink && switcher->priv->fullscreen && GTK_BIN (switcher->priv->toggle_button)->child != switcher->priv->icon)
+  {
+    if (switcher->priv->fullscreen)
+      hd_switcher_menu_dettach_button (switcher);
+    else
+      hd_switcher_menu_attach_button (switcher);
+  }
 }
 
 static void 
@@ -2242,6 +2283,8 @@ hd_switcher_menu_changed_stack_cb (HDWM *hdwm,
       }
     }	    
   }	  
+
+  g_list_free (children);
 }
 
 static void 
@@ -2320,7 +2363,27 @@ hd_switcher_menu_fullscreen_cb (HDWM *hdwm, gboolean fullscreen, HDSwitcherMenu
   }
   
   if (!fullscreen)
-    hd_switcher_menu_attach_button (switcher);	   
+  {	  
+    hd_switcher_menu_attach_button (switcher);
+
+    guint pos = 0;
+    GList *l, *apps = hd_wm_get_applications (switcher->hdwm);
+
+    for (l = apps; l != NULL; l = g_list_next (l), pos++)
+    {
+       HDWMEntryInfo *iter_info = (HDWMEntryInfo *) l->data;
+
+       if (HD_WM_IS_APPLICATION (iter_info) && switcher->priv->last_urgent_info)
+       {
+         if (iter_info == hd_wm_entry_info_get_parent (switcher->priv->last_urgent_info) ||
+	     iter_info == switcher->priv->last_urgent_info)
+           break;
+       }
+    }
+
+    if (pos <= SWITCHER_N_SLOTS)
+      hd_switcher_menu_reset_main_icon (switcher, TRUE);
+  }
 }	
 
 static void 
