@@ -305,12 +305,12 @@ hn_app_button_clicked (GtkButton *button)
 	    }
         }
 
+      toggle_button->active = !toggle_button->active;
       if (!found && app_button->priv->info != NULL)
 	new_state = (button->in_button ? GTK_STATE_PRELIGHT : GTK_STATE_ACTIVE);
       else
         {
           toggled = TRUE;
-	  toggle_button->active = !toggle_button->active;
           new_state = (button->in_button ? GTK_STATE_PRELIGHT : GTK_STATE_NORMAL);
 	}
     }
@@ -400,6 +400,7 @@ static GtkWidget *
 hn_app_button_create_menu (HNAppButton *app_button)
 {
   GtkWidget *menu;
+  GtkWidget *toplevel;
   GtkWidget *active_item = NULL;
   HDWMEntryInfo *info;
   const GList *children, *l;
@@ -444,6 +445,14 @@ hn_app_button_create_menu (HNAppButton *app_button)
 		  	       AS_MENU_ITEM_WIDTH,/*MAX (width, req.width),*/
 			       -1);
 
+  toplevel = gtk_widget_get_toplevel (GTK_WIDGET (menu));
+
+  if (GTK_IS_WIDGET (toplevel))
+  {
+    gtk_widget_set_name (toplevel, "hildon-menu-window-thumb");
+    gtk_widget_reset_rc_styles (toplevel);
+  }
+  
   return menu;
 }
 
@@ -484,7 +493,15 @@ menu_unmap_cb (GtkWidget   *widget,
       /*gtk_toggle_button_set_active (app_button->priv->prev_button, TRUE);*/
       gtk_toggle_button_toggled (app_button->priv->prev_button);
     }
-  
+  else if (!app_button->priv->prev_button)
+    {
+      /* Untoggle myself */
+      gtk_toggle_button_set_inconsistent (GTK_TOGGLE_BUTTON (app_button), TRUE);
+      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (app_button), FALSE);
+      gtk_toggle_button_toggled (GTK_TOGGLE_BUTTON (app_button));
+
+    }
+
 }
 
 static gboolean
@@ -776,6 +793,7 @@ hn_app_button_release_event (GtkWidget      *widget,
 
   gtk_widget_get_pointer(widget, &x, &y);
 
+#ifdef DEBUG
   g_debug ("pointer [%d,%d],\n"
           "allocation [%d, %d, %d, %d]",
           x, y,
@@ -783,7 +801,8 @@ hn_app_button_release_event (GtkWidget      *widget,
           widget->allocation.y,
           widget->allocation.width,
           widget->allocation.height);
-
+#endif
+  
   if(x > widget->allocation.width  || x < 0 ||
      y > widget->allocation.height || y < 0)
     {
@@ -799,7 +818,18 @@ hn_app_button_release_event (GtkWidget      *widget,
           gtk_toggle_button_set_active (priv->prev_button, TRUE);
           gtk_toggle_button_toggled (priv->prev_button);
         }
-  
+      else
+	{
+          /* Restore the previous state */
+          gtk_toggle_button_set_inconsistent (GTK_TOGGLE_BUTTON (widget), 
+			  		      !hd_wm_entry_info_is_active (app_button->priv->info));
+	  
+          gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), 
+			  		hd_wm_entry_info_is_active (app_button->priv->info));
+
+          gtk_toggle_button_toggled (GTK_TOGGLE_BUTTON (widget));
+	}
+
       /* click canceled -- we are done */
       goto out;
     }
@@ -869,7 +899,7 @@ hn_app_button_press_event (GtkWidget      *widget,
     {
       tmp_button = l->data;
 
-      if (tmp_button->active && (tmp_button != toggle_button))
+      if (tmp_button->active)
         {
           priv->prev_button = tmp_button;
           break;
